/*---------------------------------------------------------------
// HX711_ADC.h
// Arduino library for HX711 24-Bit ADC for Weigh Scales
// Olav Kallhovd sept2017
// Tested with: HX711 asian module on channel A and YZC-133 3kg load cell
// Tested with MCU: Arduino Nano
//---------------------------------------------------------------
// This is an example sketch on how to use this library for two or more HX711 modules to measure an RC model Centre of Gravity.
// Settling time (number of samples) and data filtering can be adjusted in the HX711_ADC.h file
// Diagnostics info is output on Serial & only available when the Arduino is connected to a PC
// Peter Balcombe Oct2018
*/

#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <HX711_ADC.h>

// Set up LCD address to 0x27 for a 16 chars and 2 line display (change address to suit display module in use)
LiquidCrystal_I2C lcd(0x27, 16, 2);

//HX711 constructor (dout pin, sck pin)
HX711_ADC LoadCell_1(4, 5); //HX711 sensor #1
HX711_ADC LoadCell_2(6, 7); //HX711 sensor #2

// Global variables
uint32_t displayInt = 500;  // LCD display updte period in mS
uint32_t timeNow;           // Current time
uint32_t dispLast;          // Last LCD update time
long t;                     // Last load measurement time
float d1 = 25;              // offset from wing LE to LoadCell 1
float d2 = 185;             // offset from wing LE to LoadCell 2
float w1;                   // LoadCell 1 weight
float w2;                   // LoadCell 2 weight
float w;                    // Total weight
float cog;                  // CoG offset from Ref. point

void setup()
{
  Wire.begin();
  lcd.begin();
  lcd.backlight();          // Turn on the LED backlight
  Serial.begin(9600);
  timeNow = millis();
  dispLast = timeNow;
  
  Serial.println("Wait...");
  lcd.setCursor(2, 0);
  lcd.print("Wait...");
  LoadCell_1.begin();
  LoadCell_2.begin();
  long stabilisingtime = 2000;  // tare precision can be improved by adding a few seconds of stabilising time
  byte loadcell_1_rdy = 0;
  byte loadcell_2_rdy = 0;
  while ((loadcell_1_rdy + loadcell_2_rdy) < 2)
  {
  //run startup, stabilization and tare, both modules simultaneously
      if (!loadcell_1_rdy) loadcell_1_rdy = LoadCell_1.startMultiple(stabilisingtime);
      if (!loadcell_2_rdy) loadcell_2_rdy = LoadCell_2.startMultiple(stabilisingtime);
  }

  LoadCell_1.setCalFactor(387.0); // user set calibration factor (float) for Load Cell #1
  LoadCell_2.setCalFactor(383.0); // user set calibration factor (float) for Load Cell #2

  Serial.println("Startup + tare is complete");
  lcd.setCursor(1, 0);
  lcd.print("Startup done");
  lcd.setCursor(1,1);
  lcd.print("Tare complete");
}

void loop()
{
  //update() should be called at least as often as HX711 sample rate; >10Hz@10SPS, >80Hz@80SPS
  //longer delay in sketch will reduce effective sample rate (be careful with delay() in loop)
  LoadCell_1.update();
  LoadCell_2.update();

  //get smoothed value from data set + current calibration factor
  if (millis() > t + 250)
  {
    w1 = LoadCell_1.getData();
    w2 = LoadCell_2.getData();
    Serial.print("Load_cell 1 output val: ");
    Serial.print(w1);
    Serial.print("    Load_cell 2 output val: ");
    Serial.println(w2);
    t = millis();
  }

  w = w1 + w2;  // total weight = sum of load cell values
  cog = ((d1*w1) + (d2*w2))/w;  // cog=weight dist/weight
  Serial.print("CoG distance from LE: ");
  Serial.print(cog);
  Serial.print("mm");
  Serial.print("   Total weight: ");
  Serial.print(w);
  Serial.println("g");

  timeNow = millis();
  if((timeNow - dispLast) > displayInt) // If time for new LCD update
  {
     dispLast = timeNow;
     lcd.setCursor(0, 0);     // Home lcd cursor
     lcd.print("Weight:         ");
     lcd.setCursor(8, 0);     // Start of weight value
     lcd.print(w);
     lcd.print("g");
     lcd.setCursor(0, 1);     // Start 2nd row
     lcd.print("   CoG:        ");
     lcd.setCursor(8, 1);     // Start of CoG value
     lcd.print(cog);
     lcd.print("mm");
  }

  //receive from serial terminal. (Used to reset by entering 't' on PC serial)
  if (Serial.available() > 0)
  {
     float i;
     char inByte = Serial.read();
     if (inByte == 't')
  {
        LoadCell_1.tareNoDelay();
        LoadCell_2.tareNoDelay();
     }
  }

  //check if last tare operation is complete
  if (LoadCell_1.getTareStatus() == true)
  {
     Serial.println("Tare load cell 1 complete");
  }
  if (LoadCell_2.getTareStatus() == true)
  {
     Serial.println("Tare load cell 2 complete");
  }

}
